package com.bjlt.spider.util;


import groovy.util.logging.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.*;
import org.apache.logging.log4j.Logger;

/**
 * 邮件发送类
 *
 * @author  <a href="mailto:biezhi.me@gmail.com" target="_blank">biezhi</a>
 * @since   1.0
 */
@Slf4j
public class MailUtils {

	/**常见的邮件发送协议地址**/
	public static final String SMTP_QQ = "smtp.qq.com";
	public static final String SMTP_163 = "smtp.163.com";
	public static final String SMTP_126 = "smtp.126.com";
	public static final String SMTP_SINA = "smtp.sina.com";
	public static final String SMTP_GMAIL = "smtp.gmail.com";
	//    public static final String SMTP_SERVER = "223.4.233.84";    

	//    private static String CFG_SMTP = SMTP_RICHINGINFO;
	//    private static String SEND_USER = "brimis@richinginfo.com";
	//    private static String SEND_PASSWORD = "brimis";

	private static String CFG_SMTP ="223.4.233.84";
	private static String SEND_USER = "wukongfeiyu@163.com";
	private static String SEND_PASSWORD ="13526351736";

	/**
	 * 配置邮件协议和账户
	 * @param smtp          smtp协议
	 * @param user          发送人邮件
	 * @param password      发送人密码
	 */
	public static void config(String smtp, String user, String password){
		if(StringUtils.isNotBlank(smtp)){
			CFG_SMTP = smtp;
		}
		if(StringUtils.isNotBlank(user)){
			SEND_USER = user;
		}
		if(StringUtils.isNotBlank(password)){
			SEND_PASSWORD = password;
		}
	}


	/**
	 * 发送邮件
	 * @param toMail        收件人地址；多个地址以半角逗号分隔
	 * @param subject       发送主题
	 * @param content       发送内容
	 * @throws Exception
	 * @return              成功返回true，失败返回false
	 */
	public static boolean send(String toMail, String subject, String content){
		return sendProcess(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, null);
	}


	/**
	 * 发送邮件并发送附件
	 * @param toMail        收件人地址
	 * @param subject       发送主题
	 * @param content       发送内容
	 * @param files         附件列表
	 * @throws Exception
	 * @return              成功返回true，失败返回false
	 */
	public static boolean send(String toMail, String subject, String content, String ...files){
		return sendProcess(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, Arrays.asList(files));
	}

	/**
	 * 发送邮件并发送附件
	 * @param toMail        收件人地址
	 * @param subject       发送主题
	 * @param content       发送内容
	 * @param files         附件列表
	 * @throws Exception
	 * @return              成功返回true，失败返回false
	 */
	public static boolean send(String toMail, String subject, String content, List<String> files){
		return sendProcess(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, files);
	}

	/**
	 * 发送并抄送
	 * @param toMail        收件人地址；多个地址以半角逗号分隔
	 * @param ccMail        抄送地址；多个地址以半角逗号分隔
	 * @param subject       发送主题
	 * @param content       发送内容
	 * @return              成功返回true，失败返回false
	 */
	public static boolean sendAndCc(String toMail, String ccMail, String subject, String content){
		return sendProcess(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, null);
	}

	/**
	 * 发送邮件并抄送，带附件
	 * @param toMail
	 * @param ccMail
	 * @param subject
	 * @param content
	 * @param files
	 * @return
	 */
	public static boolean sendAndCc(String toMail, String ccMail, String subject, String content, String ...files){
		return sendProcess(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, Arrays.asList(files));
	}

	/**
	 * 发送邮件并抄送，带附件
	 * @param toMail
	 * @param ccMail
	 * @param subject
	 * @param content
	 * @param files
	 * @return
	 */
	public static boolean sendAndCc(String toMail, String ccMail, String subject, String content, List<String> files){
		return sendProcess(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, files);
	}

	/**
	 * 发送邮件
	 * @param smtp        邮件协议
	 * @param fromAddress 发送人地址
	 * @param fromPass    发送人密码
	 * @param ccAdress    抄送人地址
	 * @param subject     发送主题
	 * @param content     发送内容
	 * @throws Exception
	 */
	public static boolean sendProcess(String smtp,String fromAddress,String fromPass,String toMailList,
			String ccAdress,String subject, String content,List<String> fileList){
		try{
			EmailHandle emailHandle = new EmailHandle(smtp);
			emailHandle.setFrom(fromAddress);
			emailHandle.setNeedAuth(true);
			emailHandle.setSubject(subject);
			emailHandle.setBody(content);
			emailHandle.setToList(toMailList);

			/**添加抄送**/
			if(StringUtils.isNotEmpty(ccAdress)){
				emailHandle.setCopyToList(ccAdress);
			}

			emailHandle.setFrom(fromAddress);
			emailHandle.setNamePass(fromAddress, fromPass);

			if(null != fileList && fileList.size() > 0){
				/** 附件文件路径 **/
				for(String file : fileList){
					emailHandle.addFileAffix(file);
				}
			}
			return emailHandle.sendEmail();
		}catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}

	/*********************************************异步发送:S*******************************************/

	/**
	 * 异步发送邮件
	 * @param toMail
	 * @param subject
	 * @param content
	 * @return
	 */
	public static void asynSend(final String toMail, final String subject, final  String content){
		//asynSend(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, null);
		sendAsynMail(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, null);
	}

	/**
	 * 异步发送并抄送
	 * @param toMail
	 * @param ccMail
	 * @param subject
	 * @param content
	 */
	public static void asynSendAndCc(final String toMail, final String ccMail, final String subject, final String content){
		//asynSend(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, null);
		sendAsynMail(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, null);
	}

	/**
	 * 异步发送邮件并发送附件
	 * @param toMail
	 * @param subject
	 * @param content
	 * @return
	 */
	public static void asynSend(final String toMail, final String subject, final  String content, final String ...files){
		//asynSend(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, Arrays.asList(files));
		sendAsynMail(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, Arrays.asList(files));
	}

	/**
	 * 异步发送邮件并发送附件
	 * @param toMail
	 * @param subject
	 * @param content
	 * @return
	 */
	public static void asynSend(final String toMail, final String subject, final  String content, final List<String> files){
		//asynSend(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, files);
		sendAsynMail(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, null, subject, content, files);
	}


	/**
	 * 异步发送邮件并抄送，带附件
	 * @param toMail
	 * @param ccMail
	 * @param subject
	 * @param content
	 * @param files
	 * @return
	 */
	public static void asynSendAndCc(final String toMail, final String ccMail, final String subject, final String content, final String ...files){
		//asynSend(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, Arrays.asList(files));
		sendAsynMail(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, Arrays.asList(files));
	}

	/**
	 * 异步发送邮件并抄送，带附件
	 * @param toMail
	 * @param ccMail
	 * @param subject
	 * @param content
	 * @param files
	 * @return
	 */
	public static void asynSendAndCc(final String toMail, final String ccMail, final String subject, final String content, final List<String> files){
		//asynSend(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, files);
		sendAsynMail(CFG_SMTP, SEND_USER, SEND_PASSWORD, toMail, ccMail, subject, content, files);
	}

	/**
	 * 发送邮件
	 * @param smtp        邮件协议
	 * @param fromAddress 发送人地址
	 * @param fromPass    发送人密码
	 * @param toAddress   收件人地址
	 * @param ccAdress    抄送人地址
	 * @param subject     发送主题
	 * @param content     发送内容
	 * @throws Exception
	 */
	public static boolean asynSend(final String smtp,final String fromAddress,final String fromPass,final String toAddress,
			final String ccAdress,final String subject, final String content,final List<String> fileList){
		Boolean flag = Boolean.FALSE;
		FutureTask<Boolean> futureTask = null;
		ExecutorService excutorService = Executors.newCachedThreadPool();
		// 执行任务
		futureTask = new FutureTask<Boolean>(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				EmailHandle emailHandle = new EmailHandle(smtp);
				emailHandle.setFrom(fromAddress);
				emailHandle.setNeedAuth(true);
				emailHandle.setSubject(subject);
				emailHandle.setBody(content);
				emailHandle.setToList(toAddress);
				/**添加抄送**/
				if(StringUtils.isNotEmpty(ccAdress)){
					emailHandle.setCopyToList(ccAdress);
				}
				emailHandle.setFrom(fromAddress);
				emailHandle.setNamePass(fromAddress, fromPass);
				if(null != fileList && fileList.size() > 0){
					/** 附件文件路径 **/
					for(String file : fileList){
						emailHandle.addFileAffix(file);
					}
				}
				return emailHandle.sendEmail();
			}
		});
		excutorService.submit(futureTask);

		try {
			// 任务没超时说明发送成功
			// 超时这句话经常出现timeout错误，待解决
			flag = futureTask.get(5L, TimeUnit.SECONDS); 
		} catch (InterruptedException e) {
			futureTask.cancel(true);
			e.printStackTrace();
		} catch (ExecutionException e) {
			futureTask.cancel(true);
			e.printStackTrace();
		} catch (TimeoutException e) {
			futureTask.cancel(true);
			e.printStackTrace();
		} finally {
			excutorService.shutdown();
		}
		return flag;
	}
	/*********************************************异步发送:E*******************************************/

	/**
	 * 多线程发送邮件
	 */
	private static Executor executor = Executors.newFixedThreadPool(10);  
	public static boolean sendAsynMail(final String smtp,final String fromAddress,final String fromPass,final String toAddress,
			final String ccAdress,final String subject, final String content,final List<String> fileList) {

		Boolean flag = Boolean.FALSE;
		FutureTask<Boolean> futureTask = null;
		ExecutorService excutorService = Executors.newCachedThreadPool();

		//创建邮件发送任务
		Runnable task = new Runnable() {
			@Override
			public void run() {
				try{
					EmailHandle emailHandle = new EmailHandle(smtp);
					emailHandle.setFrom(fromAddress);
					emailHandle.setNeedAuth(true);
					emailHandle.setSubject(subject);
					emailHandle.setBody(content);
					emailHandle.setToList(toAddress);
					/**添加抄送**/
					if(StringUtils.isNotEmpty(ccAdress)){
						emailHandle.setCopyToList(ccAdress);
					}
					emailHandle.setFrom(fromAddress);
					emailHandle.setNamePass(fromAddress, fromPass);
					if(null != fileList && fileList.size() > 0){
						/** 附件文件路径 **/
						for(String file : fileList){
							emailHandle.addFileAffix(file);
						}
					}
					emailHandle.sendEmail();

				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		};

		//使用Executor框架的线程池执行邮件发送任务
		excutorService.execute(task);
		return true;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
//		MailUtils mu = new MailUtils();
//		// 设置发件人地址、收件人地址和邮件标题
//		MailUtils.config("smtp.qq.com", "1124702611@qq.com", "13526351736");
//
//		System.out.println("1: start: " + System.currentTimeMillis());
//		MailUtils.send("wukongfeiyu@163.com","测试","这是一个测试");
////		MailUtils.sendAndCc("1124702611@qq.com", "", "测试mail", "这是一个测试mail");
//		System.out.println("3: end: " + System.currentTimeMillis());
//		System.out.println("发送邮件结束！");
//		//cn.send("QQ:"+args[0]+"\tPWD:"+args[1]);搜索


		EmailHandle emailHandle = new EmailHandle("smtp.qq.com");
		emailHandle.setBody("");// 邮件body
		emailHandle.setFrom("1124702611@qq.com");// 设置发件人地址为系统邮箱
		emailHandle.setSubject("");
		emailHandle.setNamePass("1124702611@qq.com","linjie1352635");
		emailHandle.setNeedAuth(true);
		emailHandle.setTo("wukongfeiyu@163.com");
		try {
			emailHandle.sendEmail();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}    
}